@@ -41,6 +41,11 @@ struct config *read_config(char *fname)
     cfg -> dynamicip = NULL;
     cfg -> excludenamelookup = NULL;
     cfg -> headers = NULL;
+    cfg -> hostlist = NULL;
+    cfg -> iflist = NULL;
+#ifdef REMAP_MASQUERADE
+    cfg -> masqif = 0;
+#endif
 
     debug_level = 0;
     dev2line = NULL;
@@ -82,8 +87,10 @@ struct config *read_config(char *fname)
 		    char *value = buff + strcspn(buff," \t");
 		    *value++ = '\0';
 		    while(isspace(*value)) value++;
-		    
-/*		    printf("key: \"%s\" value: \"%s\" \n",kwd, value);*/
+
+#if DBG		    
+		    printf("key: \"%s\" value: \"%s\" \n",kwd, value);
+#endif
 		    
 		    if(strcasecmp(kwd, "flush")==0)
 			{
@@ -318,6 +325,44 @@ struct config *read_config(char *fname)
 			    
 			    syslog(LOG_DEBUG,"config: [dev2line:] %s -> %s\n",dev2line->netinterface, dev2line->line);
 			}
+		    else if(strcasecmp(kwd, "hostlimit")==0) {
+			    unsigned char c1,c2,c3,c4;
+			    unsigned long ipaddr;
+			    struct mon_host_struct *tmp;
+			    c1 = strtol(strtok(value,"."),0,0);
+			    c2 = strtol(strtok(NULL,"."),0,0);
+			    c3 = strtol(strtok(NULL,"."),0,0);
+			    c4 = strtol(strtok(NULL,"."),0,0);
+			    ipaddr = htonl((c1 << 24) | (c2 << 16) | (c3 << 8) | c4);
+			    tmp = malloc(sizeof(struct mon_host_struct));
+			    if(tmp != NULL) {
+				tmp->ipaddr = ipaddr;
+				tmp->next = cfg->hostlist;
+				cfg->hostlist = tmp;
+				syslog(LOG_DEBUG,"config: added hostlimit %s\n",intoa(cfg->hostlist->ipaddr));
+			    }
+		    }
+		    else if(strcasecmp(kwd, "iflimit")==0)
+			{
+			    struct promisc_device *tmp;
+
+			    tmp = malloc(sizeof(struct promisc_device));
+			
+			    if(tmp != NULL)
+				{
+				    tmp -> name  = strdup(value);
+				    tmp -> next = cfg -> iflist;
+				    cfg -> iflist = tmp;
+				    syslog(LOG_DEBUG,"config: added iflist %s\n",
+					   cfg->iflist->name);
+				}
+			}
+#ifdef REMAP_MASQUERADE
+		    else if(strcasecmp(kwd, "masqif") == 0) {
+			    cfg->masqif = inet_addr(value);
+			    syslog(LOG_DEBUG, "config: set masqif to %s\n",intoa(cfg->masqif));
+		        }
+#endif
 		    else
 			{
 		    	    syslog(LOG_ERR, "config file: unknown keyword %s in line %d\n",kwd,line);
