@@ -117,6 +117,12 @@ struct promisc_device
     struct promisc_device *next;
 };
 
+/* structure for linked list of ip addresses to monitor - mk */
+struct mon_host_struct {
+    unsigned long ipaddr;
+    struct mon_host_struct *next;
+}; 
+
 struct config
 {
   char *filename;
@@ -133,7 +139,11 @@ struct config
   struct ipnetwork dynamicnet;
   struct ipnetwork *excludenamelookup;
   struct headerdat *headers;
-
+  struct mon_host_struct *hostlist;
+  struct promisc_device *iflist;
+#ifdef REMAP_MASQUERADE
+  unsigned long int masqif
+#endif
   char *dynamicip;
   int disabled; /* disabled output fields */
 };
@@ -180,13 +190,15 @@ struct ipdata
     unsigned long int src, dst;
     unsigned char proto;
     unsigned short srcport, dstport;
+#ifdef REMAP_MASQUERADE
+    unsigned long int masqsrc;
+    unsigned short msqport;
+#endif
     unsigned long int bytes;
     unsigned count;
     char *devname;
     char *user;
-    
     time_t when;
-
     struct ipdata *next;
 };
 
@@ -206,6 +218,8 @@ extern struct statistics *packets;
 
 extern volatile time_t now; /* current time */
 
+extern char *fname;
+
 /* capture-xxx.c */
 void init_capture(void);
 void do_acct(void);
