@@ -2,6 +2,9 @@
  * Network accounting
  * process.c - process packets
  * (C) 1994, 1995 Ulrich Callmeier
+ *
+ * modified April/98 by Tom Brown - tbrown@baremetal.com 
+ *  added hash table to speed up register_packet
  */
 
 #include <sys/time.h>
@@ -18,14 +21,24 @@
 #include <sys/ioctl.h>
 #include <sys/stat.h>
 
-char *rcs_revision_process_c = "$Revision: 1.7 $";
+#ifdef REMAP_MASQUERADE
+#include <net/ip_masq.h>
+#define PROC_MASQ_FILENAME "/proc/net/ip_masquerade"
+#endif
+
+char *rcs_revision_process_c = "$Revision: 1.8 $";
 
 volatile int running;
-
 struct statistics *packets;
 
-static struct ipdata *plist; /* data being collected */
-static struct ipdata *olist; /* data being written */
+#ifndef HASH_SIZE
+#define HASH_SIZE 256
+#endif
+#define HASH(s,d) ((s &0xff)^(s &0xff))
+
+
+static struct ipdata *plist[HASH_SIZE]; /* data being collected */
+static struct ipdata *olist[HASH_SIZE]; /* data being written */
 unsigned long int plistsize;
 unsigned long int olistsize;
 static volatile sig_atomic_t lck;
@@ -43,6 +56,43 @@ volatile time_t now;
 /* statistics */
 unsigned int list_compares, list_lookups;
 
+#ifdef REMAP_MASQUERADE
+void remap_masq(struct ipdata *p)
+{
+    FILE *fd;
+    char buf[257];
+    int line=0;
+    char *protocol;
+    char searchtext[20];
+    int plen;
+
+    if(p->proto == 6)
+	protocol = "TCP";
+    else if(p->proto == 17) 
+	protocol = "UDP";
+    else /* what about ICMP masquerading?? */
+	return;
+
+    plen = strlen(protocol);
+    sprintf(searchtext, "%08lX:%04X %04X",ntohl(p->dst), 
+	p->dstport, p_.srcport);
+    if((fd=fopen(PROC_MASQ_FILENAME,"r")) == NULL)
+	return;
+
+    while(fgets(buf,257,fd)) {
+	if(line++ &&
+	  (strncmp(protocol, buf, plen) == 0) &&
+	  (strncmp(searchtext, buf+plen+15, 18) == 0)) {
+	    p->msqsrc = htonl(strtoul(buf+plen+1, NULL, 16));
+	    p->msqport = strtoul(buf+plen+10, NULL, 16);
+	    fclose(fd);
+	}
+    }
+    fclose(fd);
+    return;
+}
+#endif
+
 void reopen_socket(void)
 {
     int save1, save2;
@@ -64,6 +114,8 @@ void reopen_socket(void)
 
 void do_acct()
 {
+  int i;
+
   packets = malloc(sizeof(struct statistics));
 
   if(!packets)
@@ -76,7 +128,10 @@ void do_acct()
   packets->ip_icmp = packets->ip_tcp = packets->ip_udp = packets->ip_other = 0;
   packets->notdev = packets->unenc = 0;
 
-  olist = plist = NULL;
+  for (i =0; i < HASH_SIZE; i++) {
+     olist[i] = plist[i] = NULL;
+     /*  olist = plist = NULL; */
+  }
   olistsize = plistsize = 0;
   lck = writing = dumping = 0;
 
@@ -100,19 +155,24 @@ void do_acct()
 
 void register_packet(unsigned long int src,unsigned long int dst, unsigned char proto, unsigned short srcport, unsigned short dstport, int size, char *devname, char *user)
 {
-  uid_t uid;
+  int hash_val = HASH(src,dst);
 
   if(lck==0)
     {
       struct ipdata *p;
       DEBUG(DBG_ANNOYING, sprintf(dbg, "lck = 1\n"));
       lck = 1;
-      p = plist;
+      p = plist[ hash_val ];
       list_lookups++;
       while(p)
 	{
 	  list_compares++;
-	  if( (DISABLED(DIS_PROTO) || (p->proto == proto)) && ( DISABLED(DIS_SRC) || (p->src == src)) && (DISABLED(DIS_DST) || (p->dst == dst)) &&  (DISABLED(DIS_SRCPORT) || (p->srcport == srcport)) && (DISABLED(DIS_DSTPORT) || (p->dstport == dstport)) && (strcmp(p->devname, devname)==0))
+	  if( (DISABLED(DIS_PROTO) || (p->proto == proto)) 
+           && ( DISABLED(DIS_SRC) || (p->src == src)) 
+           && (DISABLED(DIS_DST) || (p->dst == dst)) 
+           &&  (DISABLED(DIS_SRCPORT) || (p->srcport == srcport)) 
+           && (DISABLED(DIS_DSTPORT) || (p->dstport == dstport)) 
+           && (strcmp(p->devname, devname)==0))
 	    {
 	      p->bytes +=size;
 	      p->when = now;
@@ -138,6 +198,15 @@ void register_packet(unsigned long int src,unsigned long int dst, unsigned char
       p -> proto = proto;
       p -> srcport = srcport;
       p -> dstport = dstport;
+#ifdef REMAP_MASQUERADE
+      p -> msqsrc = 0;
+      p -> msqport = 0;
+      if((src == cfg->masqif) && 
+	 (srcport >= PORT_MASQ_BEGIN) &&
+	 (srcport <= PORT_MASQ_END)) {
+	    remap_masq(p);
+      }
+#endif
       p -> bytes = size;
       p -> count = 1;
       p -> devname = strdup(devname);
@@ -145,7 +214,7 @@ void register_packet(unsigned long int src,unsigned long int dst, unsigned char
 	p -> user = strdup(user);
       else
 	p -> user = NULL;
-      p -> next = plist;
+      p -> next = plist[ hash_val ];
       p -> when = now;
       
 #ifdef TCP_USER_INFO
@@ -158,7 +227,7 @@ void register_packet(unsigned long int src,unsigned long int dst, unsigned char
 	}
 #endif
       
-      plist = p;
+      plist[ hash_val ] = p;
       lck = 0;
       DEBUG(DBG_ANNOYING, sprintf(dbg, "lck = 0\n"));
     }
@@ -168,15 +237,17 @@ void register_packet(unsigned long int src,unsigned long int dst, unsigned char
     }
 }
 
-int do_write_list(FILE *f, struct ipdata *list)
+int do_write_list(FILE *f, struct ipdata *list[])
 {
     struct ipdata *p;
+    int i;
 #ifdef HUMAN_READABLE_TIME
     struct tm *tm1;
     time_t tmm;
 #endif
 
-    p = list;
+    for( i=0; i < HASH_SIZE; i++) {
+        p = list[i];
 
     while(p)
 	{
@@ -192,9 +263,20 @@ int do_write_list(FILE *f, struct ipdata *list)
     
 	    if(!DISABLED(DIS_PROTO)) if(fprintf(f, "\t%d",p->proto)<0) return 1;
 
-	    if(!DISABLED(DIS_SRC)) if(fprintf(f, "\t%s",intoa(p->src))<0) return 1;
+#ifdef REMAP_MASQUERADE
+	    if(p->msqport == 0) {
+#endif
+	      if(!DISABLED(DIS_SRC)) if(fprintf(f, "\t%s",intoa(p->src))<0) return 1;
 
-	    if(!DISABLED(DIS_SRCPORT)) if(fprintf(f, "\t%d",p->srcport)<0) return 1;
+	      if(!DISABLED(DIS_SRCPORT)) if(fprintf(f, "\t%d",p->srcport)<0) return 1;
+#ifdef REMAP_MASQUERADE
+	    }
+	    else {
+	      if(!DISABLED(DIS_SRC)) if(fprintf(f, "\t%s",intoa(p->msqsrc))<0) return 1;
+
+	      if(!DISABLED(DIS_SRCPORT)) if(fprintf(f, "\t%d",p->msqport)<0) return 1;
+	    }		
+#endif
 
 	    if(!DISABLED(DIS_DST)) if(fprintf(f, "\t%s",intoa(p->dst))<0) return 1;
 
@@ -212,6 +294,7 @@ int do_write_list(FILE *f, struct ipdata *list)
    
 	    p = p->next;
 	}
+    }
 
     if(fclose(f)<0)
 	{
@@ -320,6 +403,7 @@ void write_list(void)
 {
     FILE *f;
     char tmpn[255];
+    int i;
 
     while( (writepid = fork()) < 0) sleep(1);
     if (writepid!=0) return;
@@ -353,7 +437,9 @@ void write_list(void)
 	    exit(1);
 	}
 
-    olist = NULL;
+    /* memory leak? TAB */
+    for (i=0; i < HASH_SIZE; i++) 
+       olist[i] = NULL;
 
     unlink(tmpn);
 
@@ -366,6 +452,7 @@ void dump_curr_list(void)
 {
     FILE *f;
     char tmpn[255];
+    int i;
 
     while( (dumppid = fork()) < 0) sleep(1);
     if (dumppid!=0) return;
@@ -407,7 +494,8 @@ void dump_curr_list(void)
 	    exit(1);
 	}
 
-    plist = NULL;
+    for (i=0; i < HASH_SIZE; i++) 
+       plist[i] = NULL;
 
     unlink(tmpn);
 
@@ -537,6 +625,7 @@ void write_log(int force)
 {
   struct ipdata *p, *q;
   static struct ipdata *tlist; /* temp */
+  int i;
 
   DEBUG(DBG_STATE, sprintf(dbg, "write_log called\n"));
 
@@ -567,34 +656,36 @@ void write_log(int force)
       
       /* We build two lists:
 	 1) olist, which will be written out
-	 2) tlist, which will be the new plist
+	 2) tlist, which will be the new plist (just for this hash row)
 	 */
       
-      p = plist;
-      tlist = NULL;
-      olist = NULL;
       olistsize = 0;
       plistsize = 0;
-      
-      while(p)
-	{
-	  q = p->next;
-	  if(((now - p->when) > cfg->fdelay) || force)
-	    {
-	      p->next = olist;
-	      olist = p;
-	      olistsize++;
-	    }
-	  else
-	    {
-	      p->next = tlist;
-	      tlist = p;
-	      plistsize++;
-	    }
-	  p = q;
-	}
-      
-      plist = tlist;
+
+      for (i = 0; i < HASH_SIZE; i++) {
+	 p = plist[i];
+	 tlist = NULL;
+	 olist[i] = NULL;
+	 
+	 while(p)
+	   {
+	     q = p->next;
+	     if(((now - p->when) > cfg->fdelay) || force)
+	       {
+		 p->next = olist[i];
+		 olist[i] = p;
+		 olistsize++;
+	       }
+	     else
+	       {
+		 p->next = tlist;
+		 tlist = p;
+		 plistsize++;
+	       }
+	     p = q;
+	   }
+	 plist[i] = tlist;
+      }
       
       if(dumping == 0)
 	{
@@ -634,16 +725,19 @@ void write_log(int force)
 
       DEBUG(DBG_STATE, sprintf(dbg, "writepid is %d\n", (int) writepid));
       
-      p=olist;
-      while(p)
-	{
-	  olist = p->next;
-	  free(p->devname);
-	  if (p->user)
-	      free(p->user);
-	  free(p);
-	  p=olist;
-	}
+      for (i=0; i < HASH_SIZE; i++) {
+	 p = olist[i];
+	 while(p)
+	   {
+	     olist[i] = p->next;
+	     free(p->devname);
+	     if (p->user)
+		 free(p->user);
+	     free(p);
+	     p=olist[i];
+	   }
+      }
+        
       
       DEBUG(DBG_STATE, sprintf(dbg, "done freeing\n"));
     }
@@ -687,6 +781,53 @@ void signal_debug(int sig)
       DEBUG(DBG_STATE, sprintf(dbg, "reopening socket\n"));
       reopen_socket();
     }
+  else if(sig == SIGHUP)
+  {
+      struct ipnetwork *ip,*ip_next;
+      struct promisc_device *pr,*pr_next;
+      struct headerdat *h,*h_next;
+      struct mon_host_struct *mh,*mh_next;
+      DEBUG(DBG_STATE, sprintf(dbg, "received SIGHUP, rereading config\n"));
+      if(cfg) {
+	free(cfg->filename);
+	free(cfg->dumpname);
+	free(cfg->debugname);
+	for(ip=cfg->ignorenet;ip;ip=ip_next) {
+	    ip_next = ip->next;
+	    free(ip);
+	}
+	for(ip=cfg->dontignore;ip;ip=ip_next) {
+	    ip_next = ip->next;
+	    free(ip);
+	}
+	for(ip=cfg->excludenamelookup;ip;ip=ip_next) {
+	    ip_next = ip->next;
+	    free(ip);
+	}
+	for(pr=cfg->promisc;pr;pr=pr_next) {
+	    pr_next = pr->next;
+	    free(pr);
+	}
+	for(pr=cfg->notdev;pr;pr=pr_next) {
+	    pr_next = pr->next;
+	    free(pr);
+	}
+	for(pr=cfg->iflist;pr;pr=pr_next) {
+	    pr_next = pr->next;
+	    free(pr);
+	}
+	for(h=cfg->headers;h;h=h_next) {
+	    h_next = h->next;
+	    free(h);
+	}
+	for(mh=cfg->hostlist;mh;mh=mh_next) {
+	    mh_next = mh->next;
+	    free(mh);
+	}
+	free(cfg);
+      }
+      cfg = read_config(fname);
+  }
   else
     {
       DEBUG(DBG_SIGNAL, sprintf(dbg, "signal_debug received signal %d, this can't happen\n", sig));
